@@ -11,7 +11,6 @@ import pyarrow as pa
 import pyarrow.dataset as pds
 
 from snowflake import snowpark
-from snowflake.connector import result_batch
 from snowflake.ml.data import data_ingestor, data_source, ingestor_utils
 
 _EMPTY_RECORD_BATCH = pa.RecordBatch.from_arrays([], [])
@@ -140,16 +139,7 @@ class ArrowIngestor(data_ingestor.DataIngestor):
                 #        We may be able to optimize this by splitting the result batches into
                 #        in-memory (first batch) and file URLs (subsequent batches) and creating a
                 #        union dataset.
-                result_batches = ingestor_utils.get_dataframe_result_batches(self._session, source)
-                sources.extend(
-                    b.to_arrow(self._session.connection)
-                    if isinstance(b, result_batch.ArrowResultBatch)
-                    else b.to_arrow()
-                    for b in result_batches
-                )
-                # HACK: Mitigate typing inconsistencies in Snowpark results
-                if len(sources) > 0:
-                    sources = [_cast_if_needed(s, sources[-1].schema) for s in sources]
+                sources.append(_cast_if_needed(ingestor_utils.get_dataframe_arrow_table(self._session, source)))
                 source_format = None  # Arrow Dataset expects "None" for in-memory datasets
             else:
                 raise RuntimeError(f"Unsupported data source type: {type(source)}")
@@ -208,7 +198,15 @@ def _record_batch_to_arrays(rb: pa.RecordBatch) -> Dict[str, npt.NDArray[Any]]:
     for column, column_schema in zip(rb, rb.schema):
         # zero_copy_only=False because of nans. Ideally nans should have been imputed in feature engineering.
         array = column.to_numpy(zero_copy_only=False)
+        # If this column is a list, use the underlying type from the list values. Since this is just one column,
+        # there should only be one type within the list.
+        # TODO: Refactor to reduce data copies.
+        if isinstance(column_schema.type, pa.ListType):
+            # Update dtype of outer array:
+            array = np.array(array.tolist(), dtype=column_schema.type.value_type.to_pandas_dtype())
+
         batch_dict[column_schema.name] = array
+
     return batch_dict
 
 
