@@ -23,6 +23,11 @@ from snowflake.snowpark import session
 _TELEMETRY_PROJECT = "MLOps"
 _MODEL_TELEMETRY_SUBPROJECT = "ModelManagement"
 
+_MODEL_MONITORING_UNIMPLEMENTED_ERROR = "Model Monitoring is not implemented in python yet."
+_MODEL_MONITORING_DISABLED_ERROR = (
+    """Must enable monitoring to use this method. Please set `options={"enable_monitoring": True}` in the Registry"""
+)
+
 
 class Registry:
     def __init__(
@@ -84,7 +89,6 @@ class Registry:
                 session=session,
                 database_name=self._database_name,
                 schema_name=self._schema_name,
-                create_if_not_exists=True,  # TODO: Support static setup method to configure schema for monitoring.
                 statement_params=monitor_statement_params,
             )
 
@@ -381,34 +385,25 @@ class Registry:
     def add_monitor(
         self,
         name: str,
-        table_config: model_monitor_config.ModelMonitorTableConfig,
+        source_config: model_monitor_config.ModelMonitorSourceConfig,
         model_monitor_config: model_monitor_config.ModelMonitorConfig,
-        *,
-        add_dashboard_udtfs: bool = False,
     ) -> model_monitor.ModelMonitor:
         """Add a Model Monitor to the Registry
 
         Args:
             name: Name of Model Monitor to create
-            table_config: Configuration options of table for ModelMonitor.
+            source_config: Configuration options of table for ModelMonitor.
             model_monitor_config: Configuration options of ModelMonitor.
-            add_dashboard_udtfs: Add UDTFs useful for creating a dashboard.
 
         Returns:
             The newly added ModelMonitor object.
 
         Raises:
-            ValueError: If monitoring feature flag is not enabled.
+            ValueError: If monitoring is not enabled in the Registry.
         """
         if not self.enable_monitoring:
-            raise ValueError(
-                "Must enable monitoring in Registry to use this method. Please set the `enable_monitoring=True` option"
-            )
-
-        # TODO: Change to fully qualified source table reference to allow table to live in different DB.
-        return self._model_monitor_manager.add_monitor(
-            name, table_config, model_monitor_config, add_dashboard_udtfs=add_dashboard_udtfs
-        )
+            raise ValueError(_MODEL_MONITORING_DISABLED_ERROR)
+        return self._model_monitor_manager.add_monitor(name, source_config, model_monitor_config)
 
     @overload
     def get_monitor(self, model_version: model_version_impl.ModelVersion) -> model_monitor.ModelMonitor:
@@ -446,17 +441,14 @@ class Registry:
             The fetched ModelMonitor.
 
         Raises:
-            ValueError: If monitoring feature flag is not enabled.
-            ValueError: If neither name nor model_version specified.
+            ValueError: If monitoring is not enabled in the Registry.
         """
         if not self.enable_monitoring:
-            raise ValueError(
-                "Must enable monitoring in Registry to use this method. Please set the `enable_monitoring=True` option"
-            )
+            raise ValueError(_MODEL_MONITORING_DISABLED_ERROR)
         if name is not None:
             return self._model_monitor_manager.get_monitor(name=name)
         elif model_version is not None:
-            return self._model_monitor_manager.get_monitor_by_model_version(model_version=model_version)
+            return self._model_monitor_manager.get_monitor_by_model_version(model_version)
         else:
             raise ValueError("Must provide either `name` or `model_version` to get ModelMonitor")
 
@@ -472,12 +464,10 @@ class Registry:
             List of snowpark.Row containing metadata for each model monitor.
 
         Raises:
-            ValueError: If monitoring feature flag is not enabled.
+            ValueError: If monitoring is not enabled in the Registry.
         """
         if not self.enable_monitoring:
-            raise ValueError(
-                "Must enable monitoring in Registry to use this method. Please set the `enable_monitoring=True` option"
-            )
+            raise ValueError(_MODEL_MONITORING_DISABLED_ERROR)
         return self._model_monitor_manager.show_model_monitors()
 
     @telemetry.send_api_usage_telemetry(
@@ -492,10 +482,8 @@ class Registry:
             name: Name of the Model Monitor to delete.
 
         Raises:
-            ValueError: If monitoring feature flag is not enabled.
+            ValueError: If monitoring is not enabled in the registry.
         """
         if not self.enable_monitoring:
-            raise ValueError(
-                "Must enable monitoring in Registry to use this method. Please set the `enable_monitoring=True` option"
-            )
+            raise ValueError(_MODEL_MONITORING_DISABLED_ERROR)
         self._model_monitor_manager.delete_monitor(name)
